import { TformRecordDataType } from './RecordProxy';
export * from './utility';
/**
 * Tform rules definition given InRecord and OutRecord types.
 */
export declare type TformRules<InRecord, OutRecord> = {
    [K in keyof OutRecord]: OutRecord[K] extends object ? TformRules<InRecord, OutRecord[K]> | ((X: TformRecordDataType<InRecord>) => OutRecord[K]) : ((X: TformRecordDataType<InRecord>) => OutRecord[K]);
};
/**
 * Tform error
 */
export interface ITformError<InRecord> {
    error: Error;
    field?: string;
    recordNo: number;
    recordRaw: InRecord;
    recordId?: string;
}
export declare class Tform<InRecord, OutRecord> {
    private rules;
    private idKey;
    private errors;
    private recordCount;
    constructor(rules: TformRules<InRecord, OutRecord>, idKey?: keyof InRecord | undefined);
    private _processRules<T>(rules, record);
    transform(record: InRecord): OutRecord;
    getErrors(): Array<ITformError<InRecord>>;
    private _verifyHasID(record);
    private _extractID(record);
    private _addError(error, record, field?);
}
