"use strict";
/**
 * Copyright (C) 2018-present, Rimeto, LLC.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const RecordProxy_1 = require("./RecordProxy");
__export(require("./utility"));
class Tform {
    constructor(rules, idKey) {
        this.rules = rules;
        this.idKey = idKey;
        this.errors = [];
        this.recordCount = 0;
    }
    _processRules(rules, record) {
        const results = {};
        for (const key in rules) {
            if (!rules.hasOwnProperty(key)) {
                continue;
            }
            const rule = rules[key];
            try {
                // If rule is a function, call the rule function; otherwise traverse object.
                results[key] = _.isFunction(rule) ? rule(RecordProxy_1.RecordProxy(record)) : this._processRules(rule, record);
                if (results[key] === undefined) {
                    throw Error(`Property '${key}' of result is undefined`);
                }
                if (_.isString(results[key])) {
                    results[key] = results[key].trim();
                }
            }
            catch (e) {
                this._addError(e, record, key);
            }
        }
        return results;
    }
    transform(record) {
        this.recordCount += 1;
        this._verifyHasID(record);
        return this._processRules(this.rules, record);
    }
    getErrors() {
        return this.errors;
    }
    _verifyHasID(record) {
        if (this.idKey && !record[this.idKey]) {
            this._addError(TypeError(`Missing ID key '${this.idKey}'`), record);
        }
    }
    _extractID(record) {
        return this.idKey && record[this.idKey] !== undefined ? `${record[this.idKey]}` : undefined;
    }
    _addError(error, record, field) {
        this.errors.push({
            error,
            field,
            recordId: this._extractID(record),
            recordNo: this.recordCount,
            recordRaw: record,
        });
    }
}
exports.Tform = Tform;
